<?php


if ( ! defined( 'ABSPATH' ) ) {
    exit;
}
if ( ! class_exists( 'StockpackMedia' ) ) {
    class StockpackMedia {
        /**
         * @var Singleton The reference the *Singleton* instance of this class
         */
        private static $instance;

        /**
         * @var WeDevs_Settings_API
         */
        public $settings_api;

        /** @var StockpackAdmin */
        public $admin;

        /**
         * @var string plugin version
         */
        public $version = '2.5.7';

        /**
         * Returns the *Singleton* instance of this class.
         *
         * @return Singleton The *Singleton* instance.
         */
        public static function get_instance() {
            if ( null === self::$instance ) {
                self::$instance = new self();
            }

            return self::$instance;
        }

        /**
         * StockPack constructor.
         */
        protected function __construct() {
            add_action( 'init', array( $this, 'init' ) );
        }

        /**
         *
         */
        public function init() {
            $this->admin = \StockpackAdmin::get_instance();
            $this->filters();
            $this->actions();

        }


        /**
         *
         */
        public function enqueue() {
            global $wp_version;
            $screen = get_current_screen();
            wp_enqueue_script( 'stockpack', plugins_url( '../dist/js/stockpack.js', __FILE__ ), array(
                'media-views',
                'wp-color-picker',
                'jquery-ui-dialog'
            ), $this->version, false );

            if ( $screen->id === 'settings_page_stockpack' ) {
                wp_enqueue_script( 'stockpack-admin', plugins_url( '../dist/js/stockpack-settings.js', __FILE__ ), array( 'jquery' ), $this->version, true );
                wp_enqueue_style( 'settings-stockpack', plugins_url( '../dist/css/stockpack-settings.css', __FILE__ ), array(), $this->version );
            }
            wp_enqueue_style( 'wp-color-picker' );
            wp_enqueue_style( 'wp-jquery-ui-dialog' );
            wp_enqueue_style( 'stockpack-admin', plugins_url( '../dist/css/stockpack.css', __FILE__ ), false, $this->version );
            if ( version_compare( $wp_version, '5.3', '<=' ) ) {
                wp_enqueue_style( 'stockpack-admin-old', plugins_url( '../dist/css/stockpack-old-admin.css', __FILE__ ), false, $this->version );
            }


        }

        public function load_elementor_style() {
            wp_enqueue_style( 'stockpack-elementor', plugins_url( '../dist/css/stockpack-elementor.css', __FILE__ ), false, $this->version );
        }


        /**
         *
         */
        public function filters() {
            add_filter( 'media_view_strings', array( $this, 'tab_text' ), 10, 2 );
            add_filter( 'media_view_settings', array( $this, 'settings' ), 10, 2 );
        }

        public function actions() {
            add_action( 'admin_enqueue_scripts', array( $this, 'templates' ) );
            add_action( 'admin_enqueue_scripts', array( $this, 'enqueue' ) );
            add_action( 'elementor/editor/before_enqueue_scripts', array( $this, 'templates' ) );
            add_action( 'elementor/editor/before_enqueue_scripts', array( $this, 'enqueue' ) );
            add_action( 'elementor/editor/before_enqueue_scripts', array( $this, 'load_elementor_style' ) );
        }


        /**
         *
         */
        public function templates() {
            include_once( __DIR__ . '/../templates/attachment.php' );
            include_once( __DIR__ . '/../templates/empty.php' );
            include_once( __DIR__ . '/../templates/details.php' );
            include_once( __DIR__ . '/../templates/downloader.php' );
            include_once( __DIR__ . '/../templates/dialog.php' );
            include_once( __DIR__ . '/../templates/attribution.php' );
            include_once( __DIR__ . '/../templates/no-search.php' );
        }

        /**
         * @param $strings
         * @param $post
         *
         * @return array
         */
        public function tab_text( $strings, $post ) {
            if ( ! defined( 'STOCKPACK_DASHBOARD_URL' ) ) {
                $dashboard_url = 'https://stockpack.co';
            } else {
                $dashboard_url = STOCKPACK_DASHBOARD_URL;
            }

            $plugin_url = str_replace( '/src/', '', plugin_dir_url( __FILE__ ) );

            $strings['stockpack'] = array(
                'title'           => __( 'StockPack', 'stockpack' ),
                'button'          => __( 'Insert into post', 'stockpack' ),
                'retry'           => __( 'Retry', 'stockpack' ),
                'noMedia'         => __( 'No images have been found', 'stockpack' ),
                'noSearch'        => __( 'Start searching for images', 'stockpack' ),
                'inspiration'     => array(
                    'unsure' => __( 'Not sure what to search for?', 'stockpack' ),
                    'random' => array(
                        'item_1' => array(
                            'message' => __( 'Try "balloons" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/balloons.png',
                        ),
                        'item_2' => array(
                            'message' => __( 'Try "unicorn" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/unicorn.png',
                        ),
                        'item_3' => array(
                            'message' => __( 'Try "ice cream" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/icecream.png',
                        ),
                        'item_4' => array(
                            'message' => __( 'Try "moon" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/moon.png',
                        ),
                        'item_5' => array(
                            'message' => __( 'Try "coffee" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/coffee.png',
                        ),
                        'item_6' => array(
                            'message' => __( 'Try "tea" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/tea.png',
                        ),
                        'item_7' => array(
                            'message' => __( 'Try "lighthouse" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/lighthouse.png',
                        ),
                        'item_8' => array(
                            'message' => __( 'Try "mountains" in the top right search input', 'stockpack' ),
                            'image'   => $plugin_url . '/images/mountains.png',
                        )


                    ),
                ),
                'error'           => __( 'There has been an issue with your license key', 'stockpack' ),
                'link'            => __( 'Set license key', 'stockpack' ),
                'search'          => __( 'Search images', 'stockpack' ),
                'close'           => __( 'Close', 'stockpack' ),
                'download'        => __( 'Download', 'stockpack' ),
                'licenseAction'   => __( 'License', 'stockpack' ),
                'alreadyLicensed' => __( 'Licensed', 'stockpack' ),
                'license'         => __( 'Your license key is not valid', 'stockpack' ),
                'advanced'        => __( 'Advanced search', 'stockpack' ),
                'filters'         => [
                    'gender'      => [
                        'label'  => __( 'Gender', 'stockpack' ),
                        'female' => __( 'Female', 'stockpack' ),
                        'male'   => __( 'Male', 'stockpack' ),
                    ],
                    'provider'    => [
                        'label'   => __( 'Provider', 'stockpack' ),
                        'default' => __( 'Default', 'stockpack' ),
                        'values'  => apply_filters( 'stockpack_providers', [
                            'Adobe Stock'    => __( 'Adobe Stock', 'stockpack' ),
                            'Deposit Photos' => __( 'Deposit Photos', 'stockpack' ),
                            'Pixabay'        => __( 'Pixabay', 'stockpack' ),
                            'Unsplash'       => __( 'Unsplash', 'stockpack' ),
                        ] )
                    ],
                    'orientation' => [
                        'label'      => __( 'Orientation', 'stockpack' ),
                        'horizontal' => __( 'Horizontal', 'stockpack' ),
                        'vertical'   => __( 'Vertical', 'stockpack' ),
                    ],
                    'image_type'  => [
                        'label'        => __( 'Image type', 'stockpack' ),
                        'photo'        => __( 'Photos', 'stockpack' ),
                        'vector'       => __( 'Vectors', 'stockpack' ),
                        'illustration' => __( 'Illustrations', 'stockpack' ),
                    ],
                    'categories'  => [
                        'label'   => __( 'Category', 'stockpack' ),
                        'default' => __( 'Any Category', 'stockpack' ),
                        'values'  => [
                            'Abstract'               => __( 'Abstract', 'stockpack' ),
                            'Animals/Wildlife'       => __( 'Animals/Wildlife', 'stockpack' ),
                            'The Arts'               => __( 'The Arts', 'stockpack' ),
                            'Backgrounds/Textures'   => __( 'Backgrounds/Textures', 'stockpack' ),
                            'Beauty/Fashion'         => __( 'Beauty/Fashion', 'stockpack' ),
                            'Buildings/Landmarks'    => __( 'Buildings/Landmarks', 'stockpack' ),
                            'Business/Finance'       => __( 'Business/Finance', 'stockpack' ),
                            'Celebrities'            => __( 'Celebrities', 'stockpack' ),
                            'Editorial'              => __( 'Editorial', 'stockpack' ),
                            'Education'              => __( 'Education', 'stockpack' ),
                            'Food and Drink'         => __( 'Food and Drink', 'stockpack' ),
                            'Healthcare/Medical'     => __( 'Healthcare/Medical', 'stockpack' ),
                            'Holidays'               => __( 'Holidays', 'stockpack' ),
                            'Illustrations/Clip-Art' => __( 'Illustrations/Clip-Art', 'stockpack' ),
                            'Industrial'             => __( 'Industrial', 'stockpack' ),
                            'Interiors'              => __( 'Interiors', 'stockpack' ),
                            'Miscellaneous'          => __( 'Miscellaneous', 'stockpack' ),
                            'Nature'                 => __( 'Nature', 'stockpack' ),
                            'Objects'                => __( 'Objects', 'stockpack' ),
                            'Parks/Outdoor'          => __( 'Parks/Outdoor', 'stockpack' ),
                            'People'                 => __( 'People', 'stockpack' ),
                            'Religion'               => __( 'Religion', 'stockpack' ),
                            'Science'                => __( 'Science', 'stockpack' ),
                            'Signs/Symbols'          => __( 'Signs/Symbols', 'stockpack' ),
                            'Sports/Recreation'      => __( 'Sports/Recreation', 'stockpack' ),
                            'Technology'             => __( 'Technology', 'stockpack' ),
                            'Transportation'         => __( 'Transportation', 'stockpack' ),
                            'Vectors'                => __( 'Vectors', 'stockpack' ),
                            'Vintage'                => __( 'Vintage', 'stockpack' ),
                        ]
                    ],
                    'safe'        => [
                        'label' => __( 'Safe', 'stockpack' ),
                        'yes'   => __( 'Yes', 'stockpack' ),
                        'no'    => __( 'No', 'stockpack' ),
                    ],
                    'color'       => [
                        'text' => __( 'Color', 'stockpack' ),
                    ],

                ],
                'terms'           => [
                    'agree'          => __( 'I agree', 'stockpack' ),
                    'cancel'         => __( 'Cancel', 'stockpack' ),
                    'adobe_stock'    => [
                        'title'   => __( 'Terms agreement', 'stockpack' ),
                        'message' => __( 'Before you download the first image you need to agree to the terms of service of Adobe Stock. This will only be asked once and then we will store it for all subsequent downloads. You are a direct user of the Adobe Stock website and their terms apply. ', 'stockpack' ),
                        'link'    => 'https://stock.adobe.com/license-terms'
                    ],
                    'pixabay'        => [
                        'title'   => __( 'Terms agreement', 'stockpack' ),
                        'message' => __( 'Before you download the first image you need to agree to the terms of service of Pixabay. This will only be asked once and then we will store it for all subsequent downloads. You are a direct user of the Pixabay website and their terms apply. ', 'stockpack' ),
                        'link'    => 'https://pixabay.com/service/terms/'
                    ],
                    'unsplash'       => [
                        'title'   => __( 'Terms agreement', 'stockpack' ),
                        'message' => __( 'Before you download the first image you need to agree to the terms of service of Unsplash. This will only be asked once and then we will store it for all subsequent downloads. You are a direct user of the Unsplash website and their terms apply. ', 'stockpack' ),
                        'link'    => 'https://unsplash.com/terms'
                    ],
                    'deposit_photos' => [
                        'title'   => __( 'Terms agreement', 'stockpack' ),
                        'message' => __( 'Before you download the first image you need to agree to the terms of service of Deposit Photos. This will only be asked once and then we will store it for all subsequent downloads. You are a direct user of the Deposit Photos website and their terms apply. ', 'stockpack' ),
                        'link'    => 'https://depositphotos.com/terms-of-use.html'
                    ],
                ],
                'licensePopup'    => [
                    'proceed'         => __( 'Proceed', 'stockpack' ),
                    'checkInProgress' => __( 'Cost checking is in progress, please wait', 'stockpack' ),
                    'cancel'          => __( 'Cancel', 'stockpack' ),
                    'adobe_stock'     => [
                        'title'            => __( 'License and download', 'stockpack' ),
                        'message'          => __( 'Licensing an image will provide you a non watermarked image from Adobe Stock.', 'stockpack' ),
                        'status'           => __( 'Fetching cost...', 'stockpack' ),
                        'external'         => __( 'Adobe stock offers multiple types of accounts and multiple types of licenses. Currently only Standard is supported trough the plugin. You can use the url bellow to license this directly on the Adobe Stock website, in case your account allows that. In the case of credits missing, you can add more credits and retry.', 'stockpack' ),
                        'directLicenseUrl' => __( 'Adobe License Page', 'stockpack' ),
                    ],
                    'deposit_photos'  => [
                        'title'            => __( 'License and download', 'stockpack' ),
                        'message'          => __( 'Licensing an image will provide you a non watermarked image from Deposit Photos.', 'stockpack' ),
                        'status'           => __( 'Fetching cost...', 'stockpack' ),
                        'external'         => __( 'Currently only sizes available trough the subscription package are supported by the plugin.  You can use the url bellow to license this directly on the Deposit Photos website, in case your account allows that. If you subscription expired renew it and retry.', 'stockpack' ),
                        'directLicenseUrl' => __( 'Deposit Photos License Page', 'stockpack' ),
                    ],
                ],
                'attribution'     => [
                    'adobe_stock'    => [
                        'author_info' => __( 'Image info is available in the sidebar', 'stockpack' ),
                        'message'     => __( 'You are searching images from', 'stockpack' ),
                        'link'        => 'https://stockpack.co/recommended/adobe_stock',
                        'link_title'  => 'Adobe Stock'
                    ],
                    'default'        => [
                        'author_info' => __( 'Stock provider is being fetched...', 'stockpack' ),
                        'message'     => __( 'Loading data...', 'stockpack' ),
                        'link'        => '',
                        'link_title'  => ''
                    ],
                    'pixabay'        => [
                        'author_info' => __( 'Author info is available in the sidebar', 'stockpack' ),
                        'message'     => __( 'You are searching images from', 'stockpack' ),
                        'link'        => 'https://stockpack.co/recommended/pixabay',
                        'link_title'  => 'Pixabay'
                    ],
                    'unsplash'       => [
                        'author_info' => __( 'Author info is available in the sidebar', 'stockpack' ),
                        'message'     => __( 'You are searching images from', 'stockpack' ),
                        'link'        => 'https://stockpack.co/recommended/unsplash',
                        'link_title'  => 'Unsplash'
                    ],
                    'deposit_photos' => [
                        'author_info' => __( 'Author info is available in the sidebar', 'stockpack' ),
                        'message'     => __( 'You are searching images from', 'stockpack' ),
                        'link'        => 'https://stockpack.co/recommended/deposit_photos',
                        'link_title'  => 'Deposit Photos',
                        'warning'     => __( 'Watermarked images from Deposit Photos are allowed for testing only', 'stockpack' ),
                    ],
                ],
                'limit'           => [
                    'premium'   => [
                        'title'   => __( 'Requests limit reached', 'stockpack' ),
                        'message' => __( 'You have reached the limit for the requests. This gets reset, but if you reach this often and need a bigger solution you can contact StockPack support to talk about the options.', 'stockpack' )
                    ],
                    'anonymous' => [
                        'title'   => __( 'You need an account to continue', 'stockpack' ),
                        'message' => __( 'You have reached the limit for the requests without an account. ', 'stockpack' ),
                        'iframe'  => [
                            'src' => $dashboard_url . '/register?skim=1',
                            'id'  => 'stockpack-token-iframe',
                        ],
                        'status'  => [
                            'initial'   => __( 'Fetching remote website ...', 'stockpack' ),
                            'login'     => __( 'Sign in using the form below to continue', 'stockpack' ),
                            'register'  => __( 'Create an account in the window below to continue', 'stockpack' ),
                            'providers' => __( 'Token is being stored, you will be able to continue shortly', 'stockpack' ),
                            'elsewhere' => __( 'To get back to login you can close this window and open it again, or use the back buttons inside the window', 'stockpack' )
                        ]
                    ],
                    'free'      => [
                        'title'   => __( 'You need the premium upgrade to continue', 'stockpack' ),
                        'message' => __( 'You have reached the limit for the requests with the free account. ', 'stockpack' ),
                        'iframe'  => [
                            'src' => $dashboard_url . '/login?skim=1&upgrade=1',
                            'id'  => 'stockpack-upgrade-iframe',
                        ],
                        'status'  => [
                            'initial'   => __( 'Fetching remote website ...', 'stockpack' ),
                            'providers' => __( 'Refreshing token, you will be able to continue shortly', 'stockpack' ),
                            'login'     => __( 'Sign in using the form below to continue', 'stockpack' ),
                            'register'  => __( 'Create an account in the window below to continue', 'stockpack' ),
                            'billing'   => __( 'Once you upgrade your account you will get upgraded limits which will allow you to proceed right away', 'stockpack' ),
                            'elsewhere' => __( 'To get back to login you can close this window and open it again, or use the back buttons inside the window', 'stockpack' )
                        ]
                    ],
                    'reset'     => __( 'Reset time passed, you can try again', 'stockpack' ),
                    'contact'   => __( 'Contact', 'stockpack' ),
                    'cancel'    => __( 'Cancel', 'stockpack' )
                ],
                'filename'        => [
                    'placeholder' => __( 'Overwrite filename (No extension)' )
                ]
            );

            return $strings;
        }

        public function settings( $strings, $post ) {
            $strings['stockpack'] = array(
                'terms'                    => [
                    'adobe_stock'    => get_option( 'terms_accepted_adobe_stock', false ),
                    'pixabay'        => get_option( 'terms_accepted_pixabay', false ),
                    'unsplash'       => get_option( 'terms_accepted_unsplash', false ),
                    'deposit_photos' => get_option( 'terms_accepted_deposit_photos', false ),
                ],
                'nonce_terms'              => wp_create_nonce( 'stockpack_terms' ),
                'nonce_license_cost'       => wp_create_nonce( 'stockpack_license_cost' ),
                'nonce_query'              => wp_create_nonce( 'stockpack_query' ),
                'nonce_download'           => wp_create_nonce( 'stockpack_download' ),
                'nonce_cache'              => wp_create_nonce( 'stockpack_cache' ),
                'nonce_token'              => wp_create_nonce( 'stockpack_token' ),
                'nonce_validate'           => wp_create_nonce( 'stockpack_validate' ),
                'settings_url'             => admin_url( 'options-general.php?page=stockpack' ),
                'contact_url'              => 'https://stockpack.co/contact',
                'filename_change'          => $this->admin->get_file_name_change_setting(),
                'premium_provider_caption' => $this->admin->get_premium_providers_caption_setting()
            );

            return $strings;
        }


    }

    $GLOBALS['stockpack_media'] = StockpackMedia::get_instance();
}
